import os, re
from qgis.core import QgsVectorLayer, QgsRasterLayer

class Filter():
  """ Class to encapsulate filters behavior """
  def __init__( self ):
    pass

  def apply( self, layerPath ):
    """ To be overridden """
    pass


class NoFilter( Filter ):
  """ Dummy filter """
  def __init__( self ):
    pass

  def apply( self, layerPath ):
    """ There is no condition to be applied """
    return True


class AlphanumericFilter( Filter ):
  """ Filter based on text using a regular expression """
  def __init__( self, matchType, filterText ):
    self.matchType = matchType
    self.filterText = filterText

  def apply( self, layerPath ):
    """ Apply an alphanumeric filter """
    regExpString = ''

    if self.matchType == 'StartsWith':
      regExpString = ''.join( [ '^', self.filterText ] )
    elif self.matchType == 'EndsWith':
      regExpString = ''.join( [ self.filterText, '$' ] )
    else: # "anyPosition"
      regExpString = self.filterText

    layerBaseName = os.path.splitext( os.path.basename( layerPath ) )[0]
    return True if re.search( regExpString.decode("utf-8"), layerBaseName ) else False    


class TypeFilter( Filter ):
  """ Abstract class to define a filter based on an object's type """
  def __init__( self, itemTypes ):
    self.lstFilterItems = [] # Types to be considered as True

  def getItemType( self, layerPath ):
    """ To be overidden """
    pass

  def apply( self, layerPath ):
    """ Apply a type filter """
    itemType = self.getItemType( layerPath )
    return True if itemType in self.lstFilterItems else False


class GeometryTypeFilter( TypeFilter ):
  """ Filter based on the layer's geometry type """
  def __init__( self, itemTypes ):
    TypeFilter.__init__( self, itemTypes )
    if 'Point' in itemTypes: self.lstFilterItems.append( 0 )
    if 'Line' in itemTypes: self.lstFilterItems.append( 1 )
    if 'Polygon' in itemTypes: self.lstFilterItems.append( 2 )

  def getItemType( self, layerPath ):
    """ Get the layer's geometry type """
    provider = 'gpx' if os.path.splitext( os.path.basename( layerPath ) )[1][:4] == ".gpx" else 'ogr'
    return QgsVectorLayer( layerPath, '', provider).geometryType()

class RasterTypeFilter( TypeFilter ):
  """ Filter based on the layer's raster type """
  def __init__( self, itemTypes ):
    TypeFilter.__init__( self, itemTypes )
    if 'GrayOrUndefined' in itemTypes: self.lstFilterItems.append( 0 )
    if 'Palette' in itemTypes: self.lstFilterItems.append( 1 )
    if 'Multiband' in itemTypes: self.lstFilterItems.append( 2 )

  def getItemType( self, layerPath ):
    """ Get the layer's raster type """
    return QgsRasterLayer( layerPath, '').rasterType()


class ComposedFilter( Filter ):
  """ Abstract class to mix filters ( AlphanumericFilter + TypeFilter ) """
  def __init__( self, itemTypes, filterType, filterText ):
    self.alphanumericFilter = AlphanumericFilter( filterType, filterText ) 
    self.typeFilter = TypeFilter( itemTypes )

  def apply( self, layerPath ):
    """ Check the two filters at the same time """
    return self.alphanumericFilter.apply( layerPath ) and self.typeFilter.apply( layerPath )


class ComposedGeometricFilter( ComposedFilter ):
  """ Mixed filter for vectors """
  def __init__( self, itemTypes, filterType, filterText ):
    ComposedFilter.__init__( self, itemTypes, filterType, filterText )
    self.typeFilter = GeometryTypeFilter( itemTypes )


class ComposedRasterFilter( ComposedFilter ):
  """ Mixed filter for rasters """
  def __init__( self, itemTypes, filterType, filterText ):
    ComposedFilter.__init__( self, itemTypes, filterType, filterText )
    self.typeFilter = RasterTypeFilter( itemTypes )
    

